@@ -1,4 +1,4 @@
-/* $OpenBSD: asn_moid.c,v 1.12 2014/07/11 08:44:47 jsing Exp $ */
+/* $OpenBSD: asn_moid.c,v 1.15 2022/11/26 16:08:50 tb Exp $ */
 /* Written by Stephen Henson (steve@openssl.org) for the OpenSSL
  * project 2001.
  */
@@ -65,6 +65,8 @@
 #include <openssl/crypto.h>
 #include <openssl/x509.h>
 
+#include "asn1_local.h"
+
 /* Simple ASN1 OID module: add all objects in a given section */
 
 static int do_create(char *value, char *name);
