@@ -1,4 +1,4 @@
-/*	$OpenBSD: x509_asid.c,v 1.35 2022/07/30 17:50:17 tb Exp $ */
+/*	$OpenBSD: x509_asid.c,v 1.39 2023/02/16 08:38:17 tb Exp $ */
 /*
  * Contributed to the OpenSSL Project by the American Registry for
  * Internet Numbers ("ARIN").
@@ -73,7 +73,7 @@
 #include <openssl/x509.h>
 #include <openssl/x509v3.h>
 
-#include "x509_lcl.h"
+#include "x509_local.h"
 
 #ifndef OPENSSL_NO_RFC3779
 
@@ -191,24 +191,28 @@ d2i_ASRange(ASRange **a, const unsigned char **in, long len)
 	return (ASRange *)ASN1_item_d2i((ASN1_VALUE **)a, in, len,
 	    &ASRange_it);
 }
+LCRYPTO_ALIAS(d2i_ASRange);
 
 int
 i2d_ASRange(ASRange *a, unsigned char **out)
 {
 	return ASN1_item_i2d((ASN1_VALUE *)a, out, &ASRange_it);
 }
+LCRYPTO_ALIAS(i2d_ASRange);
 
 ASRange *
 ASRange_new(void)
 {
 	return (ASRange *)ASN1_item_new(&ASRange_it);
 }
+LCRYPTO_ALIAS(ASRange_new);
 
 void
 ASRange_free(ASRange *a)
 {
 	ASN1_item_free((ASN1_VALUE *)a, &ASRange_it);
 }
+LCRYPTO_ALIAS(ASRange_free);
 
 ASIdOrRange *
 d2i_ASIdOrRange(ASIdOrRange **a, const unsigned char **in, long len)
@@ -216,24 +220,28 @@ d2i_ASIdOrRange(ASIdOrRange **a, const unsigned char **in, long len)
 	return (ASIdOrRange *)ASN1_item_d2i((ASN1_VALUE **)a, in, len,
 	    &ASIdOrRange_it);
 }
+LCRYPTO_ALIAS(d2i_ASIdOrRange);
 
 int
 i2d_ASIdOrRange(ASIdOrRange *a, unsigned char **out)
 {
 	return ASN1_item_i2d((ASN1_VALUE *)a, out, &ASIdOrRange_it);
 }
+LCRYPTO_ALIAS(i2d_ASIdOrRange);
 
 ASIdOrRange *
 ASIdOrRange_new(void)
 {
 	return (ASIdOrRange *)ASN1_item_new(&ASIdOrRange_it);
 }
+LCRYPTO_ALIAS(ASIdOrRange_new);
 
 void
 ASIdOrRange_free(ASIdOrRange *a)
 {
 	ASN1_item_free((ASN1_VALUE *)a, &ASIdOrRange_it);
 }
+LCRYPTO_ALIAS(ASIdOrRange_free);
 
 ASIdentifierChoice *
 d2i_ASIdentifierChoice(ASIdentifierChoice **a, const unsigned char **in,
@@ -242,24 +250,28 @@ d2i_ASIdentifierChoice(ASIdentifierChoice **a, const unsigned char **in,
 	return (ASIdentifierChoice *)ASN1_item_d2i((ASN1_VALUE **)a, in, len,
 	    &ASIdentifierChoice_it);
 }
+LCRYPTO_ALIAS(d2i_ASIdentifierChoice);
 
 int
 i2d_ASIdentifierChoice(ASIdentifierChoice *a, unsigned char **out)
 {
 	return ASN1_item_i2d((ASN1_VALUE *)a, out, &ASIdentifierChoice_it);
 }
+LCRYPTO_ALIAS(i2d_ASIdentifierChoice);
 
 ASIdentifierChoice *
 ASIdentifierChoice_new(void)
 {
 	return (ASIdentifierChoice *)ASN1_item_new(&ASIdentifierChoice_it);
 }
+LCRYPTO_ALIAS(ASIdentifierChoice_new);
 
 void
 ASIdentifierChoice_free(ASIdentifierChoice *a)
 {
 	ASN1_item_free((ASN1_VALUE *)a, &ASIdentifierChoice_it);
 }
+LCRYPTO_ALIAS(ASIdentifierChoice_free);
 
 ASIdentifiers *
 d2i_ASIdentifiers(ASIdentifiers **a, const unsigned char **in, long len)
@@ -267,24 +279,28 @@ d2i_ASIdentifiers(ASIdentifiers **a, const unsigned char **in, long len)
 	return (ASIdentifiers *)ASN1_item_d2i((ASN1_VALUE **)a, in, len,
 	    &ASIdentifiers_it);
 }
+LCRYPTO_ALIAS(d2i_ASIdentifiers);
 
 int
 i2d_ASIdentifiers(ASIdentifiers *a, unsigned char **out)
 {
 	return ASN1_item_i2d((ASN1_VALUE *)a, out, &ASIdentifiers_it);
 }
+LCRYPTO_ALIAS(i2d_ASIdentifiers);
 
 ASIdentifiers *
 ASIdentifiers_new(void)
 {
 	return (ASIdentifiers *)ASN1_item_new(&ASIdentifiers_it);
 }
+LCRYPTO_ALIAS(ASIdentifiers_new);
 
 void
 ASIdentifiers_free(ASIdentifiers *a)
 {
 	ASN1_item_free((ASN1_VALUE *)a, &ASIdentifiers_it);
 }
+LCRYPTO_ALIAS(ASIdentifiers_free);
 
 /*
  * i2r method for an ASIdentifierChoice.
@@ -412,6 +428,7 @@ X509v3_asid_add_inherit(ASIdentifiers *asid, int which)
 	}
 	return (*choice)->type == ASIdentifierChoice_inherit;
 }
+LCRYPTO_ALIAS(X509v3_asid_add_inherit);
 
 /*
  * Add an ID or range to an ASIdentifierChoice.
@@ -466,6 +483,7 @@ X509v3_asid_add_id_or_range(ASIdentifiers *asid, int which, ASN1_INTEGER *min,
 	ASIdOrRange_free(aor);
 	return 0;
 }
+LCRYPTO_ALIAS(X509v3_asid_add_id_or_range);
 
 /*
  * Extract min and max values from an ASIdOrRange.
@@ -562,8 +580,8 @@ ASIdentifierChoice_is_canonical(ASIdentifierChoice *choice)
 	}
 
 	/*
-	* Check for inverted range.
-	*/
+	 * Check for inverted range.
+	 */
 	i = sk_ASIdOrRange_num(choice->u.asIdsOrRanges) - 1;
 	{
 		ASIdOrRange *a = sk_ASIdOrRange_value(choice->u.asIdsOrRanges,
@@ -594,6 +612,7 @@ X509v3_asid_is_canonical(ASIdentifiers *asid)
 	    (ASIdentifierChoice_is_canonical(asid->asnum) &&
 	     ASIdentifierChoice_is_canonical(asid->rdi)));
 }
+LCRYPTO_ALIAS(X509v3_asid_is_canonical);
 
 /*
  * Whack an ASIdentifierChoice into canonical form.
@@ -761,6 +780,7 @@ X509v3_asid_canonize(ASIdentifiers *asid)
 
 	return ASIdentifierChoice_canonize(asid->rdi);
 }
+LCRYPTO_ALIAS(X509v3_asid_canonize);
 
 /*
  * v2i method for an ASIdentifier extension.
@@ -919,6 +939,7 @@ X509v3_asid_inherits(ASIdentifiers *asid)
 
 	return 0;
 }
+LCRYPTO_ALIAS(X509v3_asid_inherits);
 
 /*
  * Figure out whether parent contains child.
@@ -992,6 +1013,7 @@ X509v3_asid_subset(ASIdentifiers *child, ASIdentifiers *parent)
 
 	return 1;
 }
+LCRYPTO_ALIAS(X509v3_asid_subset);
 
 /*
  * Validation error handling via callback.
@@ -1159,6 +1181,7 @@ X509v3_asid_validate_path(X509_STORE_CTX *ctx)
 	}
 	return asid_validate_path_internal(ctx, ctx->chain, NULL);
 }
+LCRYPTO_ALIAS(X509v3_asid_validate_path);
 
 /*
  * RFC 3779 3.3 path validation of an extension.
@@ -1176,5 +1199,6 @@ X509v3_asid_validate_resource_set(STACK_OF(X509) *chain, ASIdentifiers *ext,
 		return 0;
 	return asid_validate_path_internal(NULL, chain, ext);
 }
+LCRYPTO_ALIAS(X509v3_asid_validate_resource_set);
 
 #endif                          /* OPENSSL_NO_RFC3779 */
